import 'dart:developer';
import 'dart:io';

import 'package:finotescore/src/activity_trail.dart';
import 'package:finotescore/src/crash_detector.dart';
import 'package:finotescore/src/platform_interface.dart';
import 'package:finotescore/src/thread_block_detector.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class Fn {
  /// Should be called in your main function after WidgetsFlutterBinding.ensureInitialized().
  ///
  /// By default logging in finotes plugin is off.
  static void init() {
    if (Platform.isAndroid || Platform.isIOS) {
      FlutterError.onError = Fn._reportFatalError;
      if (Platform.isIOS) {
        PlatformInterface.establishNativeConnection();
      }
      PlatformInterface.init();
      if (Platform.isAndroid) {
        ThreadBlockDetector();
      }
    } else {
      log("Platform not yet supported");
    }
  }

  /// Activates Finotes plugin logging for debug purposes.
  ///
  /// Make sure to remove this function when generating release build.
  static void logMe() {
    if (Platform.isAndroid || Platform.isIOS) {
      PlatformInterface.logMe();
    } else {
      log("Platform not yet supported");
    }
  }

  static Future<void> _reportFatalError(
      FlutterErrorDetails flutterErrorDetails) async {
    if (Platform.isAndroid || Platform.isIOS) {
      CrashDetector().processAndReport(
          flutterErrorDetails.exception.toString(), flutterErrorDetails.stack,
          context: flutterErrorDetails.context.toString(),
          information: flutterErrorDetails.informationCollector == null
              ? []
              : flutterErrorDetails.informationCollector!(),
          fatal: true);
    } else {
      log("Platform not yet supported");
    }
  }

  /// Custom API to report errors. Use this API in ZoneGuarded code blocks
  ///
  /// See documentation for more details.
  static void reportError(Object error, StackTrace stackTrace) {
    if (Platform.isAndroid || Platform.isIOS) {
      CrashDetector()
          .processAndReport(error.toString(), stackTrace, fatal: false);
    } else {
      log("Platform not yet supported");
    }
  }

  /// API to report custom issues.
  ///
  /// [where] - Ideally pass the location where this API is executed.
  /// [description] - A short description of the issue. Custom issues with same [where] and [description] values are categorized into single ticket.
  /// [detailedDescription] -  A detailed description of the issue.
  static void reportIssue(
      String where, String description, String detailedDescription) {
    if (Platform.isAndroid || Platform.isIOS) {
      PlatformInterface.customIssue(where, description, detailedDescription);
    } else {
      log("Platform not yet supported");
    }
  }

  /// API to set custom activity trail markers.
  ///
  /// [where] - Ideally pass the location where this API is executed.
  /// [marker] -  String representation of the actvity.
  @Deprecated(
      'Use setActivityMarkerAt function with this as the first parameter')
  static void setActivityMarker(String where, String marker) {
    if (Platform.isAndroid || Platform.isIOS) {
      ActivityTrail().addMarker(where, marker);
    } else {
      log("Platform not yet supported");
    }
  }

  /// API to set custom activity trail markers.
  ///
  /// [where] - Pass 'String' as argument value.
  /// [marker] -  String representation of the activity.
  static void setActivityMarkerAt(String where, String marker) {
    if (Platform.isAndroid || Platform.isIOS) {
      ActivityTrail().addMarker(where, marker);
    } else {
      log("Platform not yet supported");
    }
  }

  /// API to raise a test issue.
  ///
  /// Once issue is raised in the dashboard, make sure to remove this API.
  static void test() {
    Fn.reportIssue(
        "Fn", "Test Issue Raised", "This issue is raised for test purposes");
  }
}
