import 'dart:developer';
import 'dart:io';

import 'package:finotescore/src/activity_trail.dart';
import 'package:finotescore/src/memory_leak.dart';
import 'package:finotescore/src/thread_block_detector.dart';
import 'package:flutter/material.dart' as s;
import 'package:stack_trace/stack_trace.dart';

/// Custom class to replace 'State' class. Helps Finotes plugin to track and record lifecycle events.
abstract class FnState<T extends s.StatefulWidget> extends s.State<T>
    with s.WidgetsBindingObserver {
  @override
  void initState() {
    super.initState();
    s.WidgetsBinding.instance.addObserver(this);

    ActivityTrail()
        .addMarker(runtimeType.toString(), "initState ${context.toString()}");
  }

  @override
  void didChangeAppLifecycleState(s.AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    ActivityTrail()
        .addMarker(runtimeType.toString(), "app lifecycle state ${state.name}");
    log('AppLifecycleState: $state');
  }

  @override
  @s.mustCallSuper
  s.Widget build(s.BuildContext context) {
    ActivityTrail()
        .addMarker(runtimeType.toString(), "buid ${context.toString()}");
    return context.widget;
  }

  @override
  void deactivate() {
    super.deactivate();
    ActivityTrail().addMarker(runtimeType.toString(), "deactivate");
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    ActivityTrail().addMarker(runtimeType.toString(), "didChangeDependencies");
  }

  @override
  void didUpdateWidget(covariant T oldWidget) {
    super.didUpdateWidget(oldWidget);
    ActivityTrail().addMarker(runtimeType.toString(), "didUpdateWidget");
  }

  @override
  void dispose() {
    super.dispose();
    s.WidgetsBinding.instance.removeObserver(this);
    ActivityTrail().addMarker(runtimeType.toString(), "dispose");
    MemoryLeak().addObject(WeakReference(this));
  }

  @override
  void setState(s.VoidCallback fn) {
    if (Platform.isAndroid)
      ThreadBlockDetector().setCurrentStackTrace(Chain.current(1).toString());
    ActivityTrail().addMarker(runtimeType.toString(), "setState");
    super.setState(fn);
  }
}

abstract class FnAutomaticKeepAliveClientMixin<T extends s.StatefulWidget>
    extends s.State<T> with s.AutomaticKeepAliveClientMixin<T> {
  @override
  void initState() {
    super.initState();
    ActivityTrail()
        .addMarker(runtimeType.toString(), "initState ${context.toString()}");
  }

  @override
  @s.mustCallSuper
  s.Widget build(s.BuildContext context) {
    super.build(context);
    ActivityTrail()
        .addMarker(runtimeType.toString(), "buid ${context.toString()}");
    return context.widget;
  }

  @override
  void deactivate() {
    super.deactivate();
    ActivityTrail().addMarker(runtimeType.toString(), "deactivate");
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    ActivityTrail().addMarker(runtimeType.toString(), "didChangeDependencies");
  }

  @override
  void didUpdateWidget(covariant T oldWidget) {
    super.didUpdateWidget(oldWidget);
    ActivityTrail().addMarker(runtimeType.toString(), "didUpdateWidget");
  }

  @override
  void dispose() {
    super.dispose();
    ActivityTrail().addMarker(runtimeType.toString(), "dispose");
  }

  @override
  void setState(s.VoidCallback fn) {
    if (Platform.isAndroid)
      ThreadBlockDetector().setCurrentStackTrace(Chain.current(1).toString());
    ActivityTrail().addMarker(runtimeType.toString(), "setState");
    super.setState(fn);
  }
}
